@@ -5,9 +5,11 @@ use std::collections::BTreeMap;
 use serde_json::Value;
 #[cfg(feature="add_rulesets")]
 use crate::strings::ERROR_SERDE_PARSE;
+#[cfg(feature="get_simple_rules_ending_with")]
+use crate::RegEx;
 #[cfg(feature="add_rulesets")]
 use std::collections::HashMap;
-#[cfg(feature="rewriter")]
+#[cfg(any(all(test,feature="get_simple_rules_ending_with",feature="add_rulesets"),feature="rewriter"))]
 use regex::Regex;
 
 #[cfg(any(feature="rewriter",feature="updater"))]
@@ -23,11 +25,6 @@ lazy_static!{
     pub(crate) static ref RULE_ACTIVE_STATES: HashMap<String, bool> = HashMap::new();
 }
 
-#[cfg(feature="rewriter")]
-lazy_static!{
-    pub(crate) static ref TRIVIAL_REGEX: Regex = Regex::new(r"^http:").unwrap();
-}
-
 
 #[cfg(feature="add_rulesets")]
 struct StaticJsonStrings {
@@ -84,6 +81,22 @@ impl Rule {
         }
     }
 
+    /// Returns a string of the from regex, regardless of the enum variant
+    pub fn from_regex(&self) -> String {
+        match self {
+            Rule::Trivial => String::from("^http:"),
+            Rule::NonTrivial(from_regex, _) => from_regex.clone()
+        }
+    }
+
+    /// Returns a string of the from regex, regardless of the enum variant
+    pub fn to(&self) -> String {
+        match self {
+            Rule::Trivial => String::from("https:"),
+            Rule::NonTrivial(_, to) => to.clone()
+        }
+    }
+
 }
 
 
@@ -146,7 +159,7 @@ impl RuleSet {
     }
 
     #[cfg(feature="add_rulesets")]
-    pub(crate) fn add_rules(&mut self, rules: &Vec<Value>) {
+    pub(crate) fn add_rules(&mut self, rules: &[Value]) {
         for rule in rules {
             if let Value::Object(rule) = rule {
                 let from = match rule.get(JSON_STRINGS.from) {
@@ -163,7 +176,7 @@ impl RuleSet {
     }
 
     #[cfg(feature="add_rulesets")]
-    pub(crate) fn add_exclusions(&mut self, exclusions: &Vec<Value>) {
+    pub(crate) fn add_exclusions(&mut self, exclusions: &[Value]) {
         let mut exclusions_vec = vec![];
         for exclusion in exclusions {
             if let Value::String(exclusion) = exclusion {
@@ -175,7 +188,7 @@ impl RuleSet {
     }
 
     #[cfg(feature="add_rulesets")]
-    pub(crate) fn add_cookierules(&mut self, cookierules: &Vec<Value>) {
+    pub(crate) fn add_cookierules(&mut self, cookierules: &[Value]) {
         let mut cookierules_vec = vec![];
 
         for cookierule in cookierules {
@@ -202,7 +215,7 @@ impl RuleSet {
     #[cfg(feature="rewriter")]
     pub(crate) fn apply(&self, url: &str) -> Option<String> {
         // If we're covered by an exclusion, return
-        if !self.exclusions.is_none() {
+        if self.exclusions.is_some() {
             let exclusions_regex = Regex::new(&self.exclusions.clone().unwrap()).unwrap();
             if exclusions_regex.is_match(&url) {
                debug!("Excluded url: {}", url);
@@ -213,7 +226,11 @@ impl RuleSet {
         for rule in self.rules.iter() {
             match rule {
                 Rule::Trivial => {
-                    return Some(TRIVIAL_REGEX.replace_all(url, "https:").to_string());
+                    let mut url = url.to_string();
+                    if url.starts_with("http:") {
+                        url.replace_range(..5, "https:");
+                    }
+                    return Some(url);
                 }
                 Rule::NonTrivial(from_regex, to) => {
                     let from_regex = Regex::new(from_regex).unwrap();
@@ -231,6 +248,7 @@ impl RuleSet {
 
 /// RuleSets consists of a tuple btreemap of rulesets, keyed by some target FQDN
 #[derive(Debug)]
+#[derive(Default)]
 pub struct RuleSets(pub BTreeMap<String, Vec<Arc<RuleSet>>>);
 
 impl RuleSets {
@@ -262,13 +280,13 @@ impl RuleSets {
     /// * `scope` - An optional string which indicates the scope of the current batch of rulesets
     /// being added (see the [ruleset update channels](https://github.com/EFForg/https-everywhere/blob/master/docs/en_US/ruleset-update-channels.md) documentation)
     #[cfg(feature="add_rulesets")]
-    pub fn add_all_from_json_string(&mut self, json_string: &String, enable_mixed_rulesets: &bool, ruleset_active_states: &HashMap<String, bool>, scope: &Option<String>) {
+    pub fn add_all_from_json_string(&mut self, json_string: &str, enable_mixed_rulesets: bool, ruleset_active_states: &HashMap<String, bool>, scope: &Option<String>) {
         let rulesets: Value = serde_json::from_str(&json_string).expect(ERROR_SERDE_PARSE);
         self.add_all_from_serde_value(rulesets, enable_mixed_rulesets, ruleset_active_states, scope);
     }
 
     #[cfg(feature="add_rulesets")]
-    pub fn add_all_from_serde_value(&mut self, rulesets: Value, enable_mixed_rulesets: &bool, ruleset_active_states: &HashMap<String, bool>, scope: &Option<String>) {
+    pub fn add_all_from_serde_value(&mut self, rulesets: Value, enable_mixed_rulesets: bool, ruleset_active_states: &HashMap<String, bool>, scope: &Option<String>) {
         let scope: Arc<Option<String>> = Arc::new(scope.clone());
 
         let mut add_one_from_json = |ruleset: Value| {
@@ -278,7 +296,7 @@ impl RuleSets {
                 let mut note = String::new();
 
                 if let Some(Value::String(default_off)) = ruleset.get(JSON_STRINGS.default_off) {
-                    if default_off != &JSON_STRINGS.user_rule {
+                    if default_off != JSON_STRINGS.user_rule {
                         default_state = false;
                     }
                     note.push_str(default_off);
@@ -286,7 +304,7 @@ impl RuleSets {
                 }
 
                 if let Some(Value::String(platform)) = ruleset.get(JSON_STRINGS.platform) {
-                    if platform == &JSON_STRINGS.mixed_content {
+                    if platform == JSON_STRINGS.mixed_content {
                         if !enable_mixed_rulesets {
                             default_state = false;
                         }
@@ -356,19 +374,45 @@ impl RuleSets {
         }
     }
 
+    /// Return a vector of `(host, ruleset, rule)`s which are active and have no exclusions, for
+    /// all hosts that are in a single ruleset, end in the given ending, and match the rule `from`
+    /// regex.  Provide the regex engine that implements the trait `RegEx` as a generic.
+    ///
+    /// # Arguments
+    ///
+    /// * `ending` - A string which indicates the target ending to search for
+    #[cfg(feature="get_simple_rules_ending_with")]
+    pub fn get_simple_rules_ending_with<T: RegEx>(&self, ending: &str) -> Vec<(&String, Arc<RuleSet>, &Rule)> {
+        let mut results = vec![];
+        for (host, ruleset) in &self.0 {
+            if host.ends_with(ending) &&
+               ruleset.len() == 1 &&
+               ruleset[0].active &&
+               ruleset[0].exclusions.is_none() {
+                for rule in &ruleset[0].rules {
+                    let from_re = T::new(&rule.from_regex());
+                    if from_re.is_match(&format!("http://{}/", host)) {
+                        results.push((host, Arc::clone(&ruleset[0]), rule));
+                    }
+                }
+            }
+        }
+        results
+    }
+
     /// Return a vector of rulesets that apply to the given host
     ///
     /// # Arguments
     ///
     /// * `host` - A string which indicates the host to search for potentially applicable rulesets
     #[cfg(feature="potentially_applicable")]
-    pub fn potentially_applicable(&self, host: &String) -> Vec<Arc<RuleSet>> {
+    pub fn potentially_applicable(&self, host: &str) -> Vec<Arc<RuleSet>> {
         let mut results = vec![];
 
         self.try_add(&mut results, host);
 
         // Ensure host is well-formed (RFC 1035)
-        if host.len() <= 0 || host.len() > 255 || host.find("..").is_some() {
+        if host.is_empty() || host.len() > 255 || host.find("..").is_some() {
             return results;
         }
 
@@ -398,7 +442,7 @@ impl RuleSets {
     }
 
     #[cfg(feature="potentially_applicable")]
-    fn try_add(&self, results: &mut Vec<Arc<RuleSet>>, host: &String) {
+    fn try_add(&self, results: &mut Vec<Arc<RuleSet>>, host: &str) {
         if self.0.contains_key(host) {
             if let Some(rulesets) = self.0.get(host) {
                 for ruleset in rulesets {
@@ -419,7 +463,7 @@ pub mod tests {
     }
 
     pub fn add_mock_rulesets(rs: &mut RuleSets) {
-        rs.add_all_from_json_string(&mock_rulesets_json(), &ENABLE_MIXED_RULESETS, &RULE_ACTIVE_STATES, &None);
+        rs.add_all_from_json_string(&mock_rulesets_json(), ENABLE_MIXED_RULESETS, &RULE_ACTIVE_STATES, &None);
     }
 
     #[test]
@@ -438,13 +482,23 @@ pub mod tests {
         assert_eq!(format!("{:?}", rs), rulesets_representation);
     }
 
+    #[test]
+    #[cfg(feature="get_simple_rules_ending_with")]
+    fn get_simple_rules_ending_with() {
+        let mut rs = RuleSets::new();
+        add_mock_rulesets(&mut rs);
+
+        assert_eq!(rs.get_simple_rules_ending_with::<Regex>(".com").len(), 2);
+        assert_eq!(rs.get_simple_rules_ending_with::<Regex>(".org").len(), 0);
+    }
+
     #[test]
     #[cfg(feature="potentially_applicable")]
     fn potentially_applicable() {
         let mut rs = RuleSets::new();
         add_mock_rulesets(&mut rs);
 
-        assert_eq!(rs.potentially_applicable(&String::from("1fichier.com")).len(), 1);
+        assert_eq!(rs.potentially_applicable("1fichier.com").len(), 1);
     }
 
     #[test]
@@ -453,10 +507,10 @@ pub mod tests {
         let mut rs = RuleSets::new();
         add_mock_rulesets(&mut rs);
 
-        assert_eq!(rs.potentially_applicable(&String::from("foo.1fichier.com")).len(), 1);
-        assert_eq!(rs.potentially_applicable(&String::from("bar.foo.1fichier.com")).len(), 1);
-        assert_eq!(rs.potentially_applicable(&String::from("foo.storage.googleapis.com")).len(), 1);
-        assert_eq!(rs.potentially_applicable(&String::from("bar.foo.storage.googleapis.com")).len(), 1);
+        assert_eq!(rs.potentially_applicable("foo.1fichier.com").len(), 1);
+        assert_eq!(rs.potentially_applicable("bar.foo.1fichier.com").len(), 1);
+        assert_eq!(rs.potentially_applicable("foo.storage.googleapis.com").len(), 1);
+        assert_eq!(rs.potentially_applicable("bar.foo.storage.googleapis.com").len(), 1);
     }
 
     #[test]
@@ -465,7 +519,7 @@ pub mod tests {
         let mut rs = RuleSets::new();
         add_mock_rulesets(&mut rs);
 
-        assert_eq!(rs.potentially_applicable(&String::from("nonmatch.example.com")).len(), 0);
+        assert_eq!(rs.potentially_applicable("nonmatch.example.com").len(), 0);
     }
 
     #[test]
